@@ -46,6 +46,7 @@ static cfg_opt_t sec_general[] =
   {
     CFG_STR("uid", "nobody", CFGF_NONE),
     CFG_STR("db_path", STATEDIR "/cache/" PACKAGE "/songs3.db", CFGF_NONE),
+    CFG_STR("db_backup_path", NULL, CFGF_NONE),
     CFG_STR("logfile", STATEDIR "/log/" PACKAGE ".log", CFGF_NONE),
     CFG_INT_CB("loglevel", E_LOG, CFGF_NONE, &cb_loglevel),
     CFG_STR("admin_password", NULL, CFGF_NONE),
@@ -159,14 +160,18 @@ static cfg_opt_t sec_airplay[] =
     CFG_BOOL("permanent", cfg_false, CFGF_NONE),
     CFG_BOOL("reconnect", cfg_false, CFGF_NODEFAULT),
     CFG_STR("password", NULL, CFGF_NONE),
+    CFG_BOOL("raop_disable", cfg_false, CFGF_NONE),
+    CFG_STR("nickname", NULL, CFGF_NONE),
     CFG_END()
   };
 
 /* Chromecast device section structure */
 static cfg_opt_t sec_chromecast[] =
   {
+    CFG_INT("max_volume", 11, CFGF_NONE),
     CFG_BOOL("exclude", cfg_false, CFGF_NONE),
     CFG_INT("offset_ms", 0, CFGF_NONE),
+    CFG_STR("nickname", NULL, CFGF_NONE),
     CFG_END()
   };
 
@@ -248,12 +253,12 @@ gid_t runas_gid;
 
 
 static void
-logger_confuse(cfg_t *cfg, const char *format, va_list args)
+logger_confuse(cfg_t *config, const char *format, va_list args)
 {
   char fmt[80];
 
-  if (cfg && cfg->name && cfg->line)
-    snprintf(fmt, sizeof(fmt), "[%s:%d] %s\n", cfg->name, cfg->line, format);
+  if (config && config->name && config->line)
+    snprintf(fmt, sizeof(fmt), "[%s:%d] %s\n", config->name, config->line, format);
   else
     snprintf(fmt, sizeof(fmt), "%s\n", format);
 
@@ -261,7 +266,7 @@ logger_confuse(cfg_t *cfg, const char *format, va_list args)
 }
 
 static int
-cb_loglevel(cfg_t *cfg, cfg_opt_t *opt, const char *value, void *result)
+cb_loglevel(cfg_t *config, cfg_opt_t *opt, const char *value, void *result)
 {
   if (strcasecmp(value, "fatal") == 0)
     *(long int *)result = E_FATAL;
