@@ -114,7 +114,7 @@
   "   parent_id      INTEGER DEFAULT 0,"		\
   "   directory_id   INTEGER DEFAULT 0,"		\
   "   query_order    VARCHAR(1024),"			\
-  "   query_limit    INTEGER DEFAULT -1,"		\
+  "   query_limit    INTEGER DEFAULT 0,"		\
   "   media_kind     INTEGER DEFAULT 1,"		\
   "   artwork_url    VARCHAR(4096) DEFAULT NULL"	\
   ");"
