@@ -262,14 +262,13 @@ httpd_fixup_uri(struct evhttp_request *req)
 /* --------------------------- REQUEST HELPERS ------------------------------ */
 
 
-
 void
-httpd_redirect_to_admin(struct evhttp_request *req)
+httpd_redirect_to(struct evhttp_request *req, const char *path)
 {
   struct evkeyvalq *headers;
 
   headers = evhttp_request_get_output_headers(req);
-  evhttp_add_header(headers, "Location", "/admin.html");
+  evhttp_add_header(headers, "Location", path);
 
   httpd_send_reply(req, HTTP_MOVETEMP, "Moved", NULL, HTTPD_SEND_NO_GZIP);
 }
@@ -301,7 +300,7 @@ httpd_request_etag_matches(struct evhttp_request *req, const char *etag)
 
   // Add cache headers to allow client side caching
   output_headers = evhttp_request_get_output_headers(req);
-  evhttp_add_header(output_headers, "Cache-Control", "private no-cache");
+  evhttp_add_header(output_headers, "Cache-Control", "private,no-cache,max-age=0");
   evhttp_add_header(output_headers, "ETag", etag);
 
   return false;
@@ -325,11 +324,12 @@ httpd_request_not_modified_since(struct evhttp_request *req, time_t mtime)
   struct evkeyvalq *output_headers;
   char last_modified[1000];
   const char *modified_since;
+  struct tm timebuf;
 
   input_headers = evhttp_request_get_input_headers(req);
   modified_since = evhttp_find_header(input_headers, "If-Modified-Since");
 
-  strftime(last_modified, sizeof(last_modified), "%a, %d %b %Y %H:%M:%S %Z", gmtime(&mtime));
+  strftime(last_modified, sizeof(last_modified), "%a, %d %b %Y %H:%M:%S %Z", gmtime_r(&mtime, &timebuf));
 
   // Return not modified, if given timestamp matches "If-Modified-Since" request header
   if (modified_since && (strcasecmp(last_modified, modified_since) == 0))
@@ -337,12 +337,28 @@ httpd_request_not_modified_since(struct evhttp_request *req, time_t mtime)
 
   // Add cache headers to allow client side caching
   output_headers = evhttp_request_get_output_headers(req);
-  evhttp_add_header(output_headers, "Cache-Control", "private no-cache");
+  evhttp_add_header(output_headers, "Cache-Control", "private,no-cache,max-age=0");
   evhttp_add_header(output_headers, "Last-Modified", last_modified);
 
   return false;
 }
 
+void
+httpd_response_not_cachable(struct evhttp_request *req)
+{
+  struct evkeyvalq *output_headers;
+
+  output_headers = evhttp_request_get_output_headers(req);
+
+  // Remove potentially set cache control headers
+  evhttp_remove_header(output_headers, "Cache-Control");
+  evhttp_remove_header(output_headers, "Last-Modified");
+  evhttp_remove_header(output_headers, "ETag");
+
+  // Tell clients that they are not allowed to cache this response
+  evhttp_add_header(output_headers, "Cache-Control", "no-store");
+}
+
 static void
 serve_file(struct evhttp_request *req, const char *uri)
 {
@@ -373,83 +389,68 @@ serve_file(struct evhttp_request *req, const char *uri)
       return;
     }
 
-  ret = lstat(path, &sb);
-  if (ret < 0)
+  if (!realpath(path, deref))
     {
-      DPRINTF(E_WARN, L_HTTPD, "Could not lstat() %s: %s\n", path, strerror(errno));
+      DPRINTF(E_LOG, L_HTTPD, "Could not dereference %s: %s\n", path, strerror(errno));
+
+      httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
+      return;
+    }
+
+  if (strlen(deref) >= PATH_MAX)
+    {
+      DPRINTF(E_LOG, L_HTTPD, "Dereferenced path exceeds PATH_MAX: %s\n", path);
 
       httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
 
       return;
     }
 
-  if (S_ISLNK(sb.st_mode))
+  ret = lstat(deref, &sb);
+  if (ret < 0)
     {
-      if (!realpath(path, deref))
-	{
-	  DPRINTF(E_LOG, L_HTTPD, "Could not dereference %s: %s\n", path, strerror(errno));
+      DPRINTF(E_WARN, L_HTTPD, "Could not lstat() %s: %s\n", deref, strerror(errno));
 
-	  httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
+      httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
 
-	  return;
-	}
-
-      if (strlen(deref) + 1 > PATH_MAX)
-	{
-	  DPRINTF(E_LOG, L_HTTPD, "Dereferenced path exceeds PATH_MAX: %s\n", path);
-
-	  httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
-
-	  return;
-	}
-
-      strcpy(path, deref);
-
-      ret = stat(path, &sb);
-      if (ret < 0)
-	{
-	  DPRINTF(E_LOG, L_HTTPD, "Could not stat() %s: %s\n", path, strerror(errno));
-
-	  httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
-
-	  return;
-	}
+      return;
     }
 
   if (S_ISDIR(sb.st_mode))
     {
       slashed = (path[strlen(path) - 1] == '/');
+      strncat(path, ((slashed) ? "index.html" : "/index.html"), sizeof(path) - strlen(path) - 1);
 
-      ret = snprintf(deref, sizeof(deref), "%s%sindex.html", path, (slashed) ? "" : "/");
-      if ((ret < 0) || (ret >= sizeof(deref)))
+      if (!realpath(path, deref))
         {
-	  DPRINTF(E_LOG, L_HTTPD, "Redirection URL exceeds buffer length\n");
+          DPRINTF(E_LOG, L_HTTPD, "Could not dereference %s: %s\n", path, strerror(errno));
 
+          httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
+          return;
+        }
+
+      if (strlen(deref) >= PATH_MAX)
+        {
+          DPRINTF(E_LOG, L_HTTPD, "Dereferenced path exceeds PATH_MAX: %s\n", path);
+
+          httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
+
+          return;
+        }
+
+      ret = stat(deref, &sb);
+      if (ret < 0)
+        {
+	  DPRINTF(E_LOG, L_HTTPD, "Could not stat() %s: %s\n", path, strerror(errno));
 	  httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
 	  return;
 	}
-
-      strcpy(path, deref);
-
-      ret = stat(path, &sb);
-      if (ret < 0)
-        {
-	  if (strcmp(uri, "/") == 0)
-	    {
-	      httpd_redirect_to_admin(req);
-	      return;
-	    }
-	  else
-	    {
-	      DPRINTF(E_LOG, L_HTTPD, "Could not stat() %s: %s\n", path, strerror(errno));
-	      httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
-	      return;
-	    }
-	}
     }
 
-  if (path_is_legal(path) != 0)
+  if (path_is_legal(deref) != 0)
     {
+      DPRINTF(E_WARN, L_HTTPD, "Access to file outside the web root dir forbidden: %s\n", deref);
+
       httpd_send_error(req, 403, "Forbidden");
 
       return;
@@ -470,10 +471,10 @@ serve_file(struct evhttp_request *req, const char *uri)
       return;
     }
 
-  fd = open(path, O_RDONLY);
+  fd = open(deref, O_RDONLY);
   if (fd < 0)
     {
-      DPRINTF(E_LOG, L_HTTPD, "Could not open %s: %s\n", path, strerror(errno));
+      DPRINTF(E_LOG, L_HTTPD, "Could not open %s: %s\n", deref, strerror(errno));
 
       httpd_send_error(req, HTTP_NOTFOUND, "Not Found");
       evbuffer_free(evbuf);
@@ -816,14 +817,14 @@ httpd_gen_cb(struct evhttp_request *req, void *arg)
   if (!uri)
     {
       DPRINTF(E_WARN, L_HTTPD, "No URI in request\n");
-      httpd_redirect_to_admin(req);
+      httpd_redirect_to(req, "/");
       return;
     }
 
   parsed = httpd_uri_parse(uri);
   if (!parsed || !parsed->path)
     {
-      httpd_redirect_to_admin(req);
+      httpd_redirect_to(req, "/");
       goto out;
     }
 
