@@ -30,7 +30,7 @@
 #include <sys/types.h>
 #include <limits.h>
 
-#include <mxml.h>
+#include "mxml-compat.h"
 
 #include "httpd_rsp.h"
 #include "logger.h"
@@ -44,7 +44,6 @@
 #define RSP_VERSION "1.0"
 #define RSP_XML_ROOT "?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?"
 
-
 #define F_FULL     (1 << 0)
 #define F_BROWSE   (1 << 1)
 #define F_ID       (1 << 2)
@@ -57,6 +56,8 @@ struct field_map {
   int flags;
 };
 
+static char rsp_filter_files[32];
+
 static const struct field_map pl_fields[] =
   {
     { "id",           dbpli_offsetof(id),           F_ALWAYS },
@@ -209,6 +210,7 @@ static int
 query_params_set(struct query_params *qp, struct httpd_request *hreq)
 {
   const char *param;
+  char *filter;
   int ret;
 
   qp->offset = 0;
@@ -250,6 +252,15 @@ query_params_set(struct query_params *qp, struct httpd_request *hreq)
 	DPRINTF(E_LOG, L_RSP, "Ignoring improper RSP query\n");
     }
 
+  // Always filter to include only files (not streams and Spotify)
+  if (qp->filter)
+    filter = safe_asprintf("%s AND %s", qp->filter, rsp_filter_files);
+  else
+    filter = strdup(rsp_filter_files);
+
+  free(qp->filter);
+  qp->filter = filter;
+
   return 0;
 }
 
@@ -316,9 +327,9 @@ rsp_reply_info(struct httpd_request *hreq)
   mxml_node_t *node;
   cfg_t *lib;
   char *library;
-  int songcount;
+  uint32_t songcount;
 
-  songcount = db_files_get_count();
+  db_files_get_count(&songcount, NULL, rsp_filter_files);
 
   lib = cfg_getsec(cfg, "library");
   library = cfg_getstr(lib, "name");
@@ -347,7 +358,7 @@ rsp_reply_info(struct httpd_request *hreq)
 
   /* Info block */
   node = mxmlNewElement(info, "count");
-  mxmlNewTextf(node, 0, "%d", songcount);
+  mxmlNewTextf(node, 0, "%d", (int)songcount);
 
   node = mxmlNewElement(info, "rsp-version");
   mxmlNewText(node, 0, RSP_VERSION);
@@ -377,7 +388,7 @@ rsp_reply_db(struct httpd_request *hreq)
   int i;
   int ret;
 
-  memset(&qp, 0, sizeof(struct db_playlist_info));
+  memset(&qp, 0, sizeof(struct query_params));
 
   qp.type = Q_PL;
   qp.idx_type = I_NONE;
@@ -414,8 +425,12 @@ rsp_reply_db(struct httpd_request *hreq)
   mxmlNewTextf(node, 0, "%d", qp.results);
 
   /* Playlists block (all playlists) */
-  while (((ret = db_query_fetch_pl(&qp, &dbpli, 1)) == 0) && (dbpli.id))
+  while (((ret = db_query_fetch_pl(&qp, &dbpli)) == 0) && (dbpli.id))
     {
+      // Skip non-local playlists, can't be streamed to the device
+      if (!dbpli.path || dbpli.path[0] != '/')
+	continue;
+
       /* Playlist block (one playlist) */
       pl = mxmlNewElement(pls, "playlist");
 
@@ -793,7 +808,17 @@ rsp_stream(struct httpd_request *hreq)
   return 0;
 }
 
-
+// Sample RSP requests:
+//  /rsp/info
+//  /rsp/db
+//  /rsp/db/13?type=id
+//  /rsp/db/0/artist?type=browse
+//  /rsp/db/0/album?query=artist%3D%22Sting%22&type=browse
+//  /rsp/db/0?query=artist%3D%22Sting%22%20and%20album%3D%22...All%20This%20Time%22&type=browse
+//  /rsp/db/0?query=id%3D36364&type=full
+//  /rsp/stream/36364
+//  /rsp/db/0?query=id%3D36365&type=full
+//  /rsp/stream/36365
 static struct httpd_uri_map rsp_handlers[] =
   {
     {
@@ -871,6 +896,8 @@ rsp_init(void)
   int i;
   int ret;
 
+  snprintf(rsp_filter_files, sizeof(rsp_filter_files), "f.data_kind = %d", DATA_KIND_FILE);
+
   for (i = 0; rsp_handlers[i].handler; i++)
     {
       ret = regcomp(&rsp_handlers[i].preg, rsp_handlers[i].regexp, REG_EXTENDED | REG_NOSUB);
