@@ -370,6 +370,12 @@ handle_progress(struct input_metadata *m, char *progress)
   DPRINTF(E_DBG, L_PLAYER, "Received Shairport metadata progress: %" PRIi64 "/%" PRIi64 "/%" PRIi64 " => %d/%u ms\n", start, pos, end, m->pos_ms, m->len_ms);
 }
 
+static void
+handle_flush(void)
+{
+  player_playback_flush();
+}
+
 static void
 handle_volume(const char *volume)
 {
@@ -453,6 +459,8 @@ handle_picture(struct input_metadata *m, uint8_t *data, int data_len)
       return;
     }
 
+  DPRINTF(E_DBG, L_PLAYER, "Wrote pipe artwork to '%s'\n", pipe_metadata.pict_tmpfile_path);
+
   m->artwork_url = safe_asprintf("file:%s", pipe_metadata.pict_tmpfile_path);
 }
 
@@ -504,6 +512,12 @@ handle_item(struct input_metadata *m, const char *item)
       goto out_error;
     }
 
+  if (code == dmapval("pfls"))
+    {
+      handle_flush();
+      goto out_nothing;
+    }
+
   if (code == dmapval("asal"))
     dstptr = &m->album;
   else if (code == dmapval("asar"))
