@@ -338,10 +338,8 @@ file_type_get(const char *path) {
   if ((strcasecmp(ext, ".jpg") == 0) || (strcasecmp(ext, ".png") == 0))
     return FILE_IGNORE;
 
-#ifdef ITUNES
   if (strcasecmp(ext, ".xml") == 0)
     return FILE_ITUNES;
-#endif
 
   if (strcasecmp(ext, ".remote") == 0)
     return FILE_CTRL_REMOTE;
@@ -486,10 +484,8 @@ process_playlist(char *file, time_t mtime, int dir_id)
   ft = file_type_get(file);
   if (ft == FILE_PLAYLIST)
     scan_playlist(file, mtime, dir_id);
-#ifdef ITUNES
   else if (ft == FILE_ITUNES)
     scan_itunes_itml(file, mtime, dir_id);
-#endif
 }
 
 /* If we found a control file we want to kickoff some action */
@@ -640,12 +636,12 @@ process_regular_file(const char *file, struct stat *sb, int type, int flags, int
 
 /* Thread: scan */
 static void
-process_file(char *file, struct stat *sb, int type, int flags, int dir_id)
+process_file(char *file, struct stat *sb, enum file_type file_type, int scan_type, int flags, int dir_id)
 {
-  switch (file_type_get(file))
+  switch (file_type)
     {
       case FILE_REGULAR:
-	process_regular_file(file, sb, type, flags, dir_id);
+	process_regular_file(file, sb, scan_type, flags, dir_id);
 
 	counter++;
 
@@ -826,7 +822,8 @@ process_directory(char *path, int parent_id, int flags)
   int is_link;
   int follow_symlinks;
   struct watch_info wi;
-  int type;
+  int scan_type;
+  enum file_type file_type;
   char virtual_path[PATH_MAX];
   int dir_id;
   int ret;
@@ -854,13 +851,13 @@ process_directory(char *path, int parent_id, int flags)
     }
 
   /* Check if compilation and/or podcast directory */
-  type = 0;
+  scan_type = 0;
   if (check_speciallib(path, "compilations"))
-    type |= F_SCAN_TYPE_COMPILATION;
+    scan_type |= F_SCAN_TYPE_COMPILATION;
   if (check_speciallib(path, "podcasts"))
-    type |= F_SCAN_TYPE_PODCAST;
+    scan_type |= F_SCAN_TYPE_PODCAST;
   if (check_speciallib(path, "audiobooks"))
-    type |= F_SCAN_TYPE_AUDIOBOOK;
+    scan_type |= F_SCAN_TYPE_AUDIOBOOK;
 
   follow_symlinks = cfg_getbool(cfg_getsec(cfg, "library"), "follow_symlinks");
 
@@ -892,6 +889,10 @@ process_directory(char *path, int parent_id, int flags)
 	  continue;
 	}
 
+      file_type = file_type_get(entry);
+      if (file_type == FILE_IGNORE)
+	continue;
+
       ret = read_attributes(resolved_path, entry, &sb, &is_link);
       if (ret < 0)
 	{
@@ -913,7 +914,7 @@ process_directory(char *path, int parent_id, int flags)
       else if (!(flags & F_SCAN_FAST))
 	{
 	  if (S_ISREG(sb.st_mode) || S_ISFIFO(sb.st_mode))
-	    process_file(resolved_path, &sb, type, flags, dir_id);
+	    process_file(resolved_path, &sb, file_type, scan_type, flags, dir_id);
 	  else
 	    DPRINTF(E_LOG, L_SCAN, "Skipping %s, not a directory, symlink, pipe nor regular file\n", entry);
 	}
@@ -1032,6 +1033,7 @@ bulk_scan(int flags)
   dirstack = NULL;
 
   lib = cfg_getsec(cfg, "library");
+  counter = 0;
 
   ndirs = cfg_size(lib, "directories");
   for (i = 0; i < ndirs; i++)
@@ -1061,7 +1063,6 @@ bulk_scan(int flags)
 	  continue;
 	}
 
-      counter = 0;
       db_transaction_begin();
 
       process_directories(deref, parent_id, flags);
@@ -1136,7 +1137,7 @@ process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
   int parent_id;
   int fd;
 
-  DPRINTF(E_DBG, L_SCAN, "Directory event: 0x%x, cookie 0x%x, wd %d\n", ie->mask, ie->cookie, wi->wd);
+  DPRINTF(E_DBG, L_SCAN, "Directory event: 0x%08x, cookie 0x%08x, wd %d\n", ie->mask, ie->cookie, wi->wd);
 
   if (ie->mask & IN_UNMOUNT)
     {
@@ -1245,7 +1246,7 @@ process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
 	}
       else if (ret < 0)
 	{
-	  DPRINTF(E_LOG, L_SCAN, "Directory access to '%s' achieved\n", path);
+	  DPRINTF(E_INFO, L_SCAN, "Directory access to '%s' achieved\n", path);
 
 	  ie->mask |= IN_CREATE;
 	}
@@ -1278,14 +1279,19 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
   char *file = path;
   char resolved_path[PATH_MAX];
   char dir_vpath[PATH_MAX];
-  int type;
+  enum file_type file_type;
+  int scan_type;
   int i;
   int dir_id;
   int fd;
   char *ptr;
   int ret;
 
-  DPRINTF(E_DBG, L_SCAN, "File event: 0x%x, cookie 0x%x, wd %d\n", ie->mask, ie->cookie, wi->wd);
+  DPRINTF(E_DBG, L_SCAN, "File event: 0x%08x, cookie 0x%08x, wd %d\n", ie->mask, ie->cookie, wi->wd);
+
+  file_type = file_type_get(path);
+  if (file_type == FILE_IGNORE)
+    return;
 
   path_hash = djb_hash(path, strlen(path));
 
@@ -1423,13 +1429,13 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
           return;
         }
 
-      type = 0;
+      scan_type = 0;
       if (check_speciallib(path, "compilations"))
-	type |= F_SCAN_TYPE_COMPILATION;
+	scan_type |= F_SCAN_TYPE_COMPILATION;
       if (check_speciallib(path, "podcasts"))
-	type |= F_SCAN_TYPE_PODCAST;
+	scan_type |= F_SCAN_TYPE_PODCAST;
       if (check_speciallib(path, "audiobooks"))
-	type |= F_SCAN_TYPE_AUDIOBOOK;
+	scan_type |= F_SCAN_TYPE_AUDIOBOOK;
 
       dir_id = get_parent_dir_id(file);
 
@@ -1441,7 +1447,7 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
 	}
       else if (S_ISREG(sb.st_mode) || S_ISFIFO(sb.st_mode))
 	{
-	  process_file(resolved_path, &sb, type, 0, dir_id);
+	  process_file(resolved_path, &sb, file_type, scan_type, 0, dir_id);
 	}
       else
 	DPRINTF(E_LOG, L_SCAN, "Skipping %s, not a directory, symlink, pipe nor regular file\n", resolved_path);
@@ -1732,47 +1738,11 @@ filescanner_fullrescan()
   return 0;
 }
 
-static void
-map_media_file_to_queue_item(struct db_queue_item *queue_item, struct media_file_info *mfi)
-{
-  memset(queue_item, 0, sizeof(struct db_queue_item));
-
-  if (mfi->id)
-    queue_item->file_id = mfi->id;
-  else
-    queue_item->file_id = DB_MEDIA_FILE_NON_PERSISTENT_ID;
-
-  queue_item->title = safe_strdup(mfi->title);
-  queue_item->artist = safe_strdup(mfi->artist);
-  queue_item->album_artist = safe_strdup(mfi->album_artist);
-  queue_item->album = safe_strdup(mfi->album);
-  queue_item->genre = safe_strdup(mfi->genre);
-  queue_item->artist_sort = safe_strdup(mfi->artist_sort);
-  queue_item->album_artist_sort = safe_strdup(mfi->album_artist_sort);
-  queue_item->album_sort = safe_strdup(mfi->album_sort);
-  queue_item->path = safe_strdup(mfi->path);
-  queue_item->virtual_path = safe_strdup(mfi->virtual_path);
-  queue_item->data_kind = mfi->data_kind;
-  queue_item->media_kind = mfi->media_kind;
-  queue_item->song_length = mfi->song_length;
-  queue_item->seek = mfi->seek;
-  queue_item->songalbumid = mfi->songalbumid;
-  queue_item->time_modified = mfi->time_modified;
-  queue_item->year = mfi->year;
-  queue_item->track = mfi->track;
-  queue_item->disc = mfi->disc;
-  //queue_item->artwork_url
-  queue_item->type = safe_strdup(mfi->type);
-  queue_item->channels = mfi->channels;
-  queue_item->samplerate = mfi->samplerate;
-  queue_item->bitrate = mfi->bitrate;
-}
-
 static int
 queue_item_stream_add(const char *path, int position, char reshuffle, uint32_t item_id, int *count, int *new_item_id)
 {
   struct media_file_info mfi;
-  struct db_queue_item item;
+  struct db_queue_item qi;
   struct db_queue_add_info queue_add_info;
   int ret;
 
@@ -1780,12 +1750,12 @@ queue_item_stream_add(const char *path, int position, char reshuffle, uint32_t i
 
   scan_metadata_stream(&mfi, path);
 
-  map_media_file_to_queue_item(&item, &mfi);
+  db_queue_item_from_mfi(&qi, &mfi);
 
   ret = db_queue_add_start(&queue_add_info, position);
   if (ret == 0)
     {
-      ret = db_queue_add_item(&queue_add_info, &item);
+      ret = db_queue_add_next(&queue_add_info, &qi);
       ret = db_queue_add_end(&queue_add_info, reshuffle, item_id, ret);
       if (ret == 0)
 	{
@@ -1796,7 +1766,7 @@ queue_item_stream_add(const char *path, int position, char reshuffle, uint32_t i
 	}
     }
 
-  free_queue_item(&item, 1);
+  free_queue_item(&qi, 1);
   free_mfi(&mfi, 1);
 
   return 0;
