@@ -25,6 +25,11 @@
 # define MAX(a, b) (((a) > (b)) ? (a) : (b))
 #endif
 
+// If you have something like "#define MYNUMBER 3" then NTOSTR(MYNUMBER) will be
+// the string value, so "3".
+#define NTOSTR_HELPER(x) #x
+#define NTOSTR(x) NTOSTR_HELPER(x)
+
 
 // Remember to adjust quality_is_equal() if adding elements
 struct media_quality {
@@ -145,6 +150,9 @@ b64_encode(const uint8_t *src, int srclen);
 uint64_t
 murmur_hash64(const void *key, int len, uint32_t seed);
 
+void
+uuid_make(char *str);
+
 int
 linear_regression(double *m, double *b, double *r, const double *x, const double *y, int n);
 
