@@ -40,6 +40,7 @@
 #include "outputs.h"
 
 extern struct output_definition output_raop;
+extern struct output_definition output_airplay;
 extern struct output_definition output_streaming;
 extern struct output_definition output_dummy;
 extern struct output_definition output_fifo;
@@ -59,6 +60,7 @@ extern struct event_base *evbase_player;
 // Must be in sync with enum output_types
 static struct output_definition *outputs[] = {
     &output_raop,
+    &output_airplay,
     &output_streaming,
     &output_dummy,
     &output_fifo,
@@ -532,9 +534,6 @@ vol_adjust(void)
 
   for (device = outputs_device_list; device; device = device->next)
     {
-      if (!OUTPUTS_DEVICE_DISPLAY_SELECTED(device) && (device->volume > outputs_master_volume))
-	device->volume = outputs_master_volume;
-
       device->relvol = vol_to_rel(device->volume, outputs_master_volume);
     }
 
@@ -704,6 +703,21 @@ outputs_device_add(struct output_device *add, bool new_deselect)
 	break;
     }
 
+  // This is relevant for Airplay 1 and 2 where the same device can support both
+  if (device && device->type != add->type)
+    {
+      if (outputs_priority(device) < outputs_priority(add))
+	{
+	  DPRINTF(E_DBG, L_PLAYER, "Ignoring type %s for device '%s', will use type %s\n", add->type_name, add->name, device->type_name);
+	  outputs_device_free(add);
+	  return NULL;
+	}
+
+      // Remove existing device, higher priority device will be added below
+      outputs_device_remove(device);
+      device = NULL;
+    }
+
   // New device
   if (!device)
     {
@@ -802,7 +816,7 @@ outputs_device_remove(struct output_device *remove)
   if (ret < 0)
     DPRINTF(E_LOG, L_PLAYER, "Could not save state for %s device '%s'\n", remove->type_name, remove->name);
 
-  DPRINTF(E_INFO, L_PLAYER, "Removing %s device '%s'; stopped advertising\n", remove->type_name, remove->name);
+  DPRINTF(E_INFO, L_PLAYER, "Removing %s device '%s'\n", remove->type_name, remove->name);
 
   if (!prev)
     outputs_device_list = remove->next;
@@ -815,12 +829,23 @@ outputs_device_remove(struct output_device *remove)
 }
 
 void
-outputs_device_select(struct output_device *device)
+outputs_device_select(struct output_device *device, int max_volume)
 {
   device->selected = 1;
   device->prevent_playback = 0;
   device->busy = 0;
 
+  // The purpose of this is to cap the volume for a newly selected device. It is
+  // used by the player to avoid this scenario:
+  // 1 Play on two speakers, say Kitchen (100) and Office (75), master is 100
+  // 2 Disable Office, reduce master to 25, Kitchen is now 25, Office is still 75
+  // 3 Turn on Office, it now blasts at 75
+  // We could avoid this by reducing the unselected Office in step 2, but that
+  // leads to issue #1077, where volumes of unselected devices go to 0 (e.g. by
+  // reducing master to 0 and then increasing again -> unselected stays at 0).
+  if (max_volume >= 0 && device->volume > max_volume)
+    device->volume = max_volume;
+
   vol_adjust();
 }
 
@@ -1010,19 +1035,27 @@ outputs_start(output_status_cb started_cb, output_status_cb stopped_cb, bool onl
 {
   struct output_device *device;
   int pending = 0;
-  int ret = -1;
+  int ret;
 
   for (device = outputs_device_list; device; device = device->next)
     {
       if (device->selected)
-	ret = outputs_device_start(device, started_cb, only_probe);
-      else
-	ret = outputs_device_stop(device, stopped_cb);
-
-      if (ret < 0)
 	continue;
 
-      pending += ret;
+      ret = outputs_device_stop(device, stopped_cb);
+      if (ret > 0)
+	pending += ret;
+    }
+
+  ret = 0; // We don't care about devices that returned an error on stop
+  for (device = outputs_device_list; device; device = device->next)
+    {
+      if (!device->selected)
+	continue;
+
+      ret = outputs_device_start(device, started_cb, only_probe);
+      if (ret > 0)
+	pending += ret;
     }
 
   return (pending > 0) ? pending : ret;
@@ -1037,9 +1070,6 @@ outputs_stop(output_status_cb cb)
 
   for (device = outputs_device_list; device; device = device->next)
     {
-      if (!device->session)
-	continue;
-
       ret = outputs_device_stop(device, cb);
       if (ret < 0)
 	continue;
@@ -1213,6 +1243,11 @@ outputs_init(void)
 	  return -1;
 	}
 
+      if (outputs[i]->disabled)
+	{
+	  continue;
+	}
+
       if (!outputs[i]->init)
 	{
 	  no_output = 0;
