@@ -65,7 +65,7 @@
 
 #ifdef HAVE_TIMERFD
 # include <sys/timerfd.h>
-#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
+#else
 # include <signal.h>
 #endif
 
@@ -77,6 +77,7 @@
 #include "db.h"
 #include "logger.h"
 #include "conffile.h"
+#include "settings.h"
 #include "misc.h"
 #include "player.h"
 #include "worker.h"
@@ -125,6 +126,11 @@
 // know if they should only probe the device, or fully start it.
 #define PLAYER_ONLY_PROBE (player_state != PLAY_PLAYING)
 
+// Name of settings used by player
+#define PLAYER_SETTINGS_MODE_REPEAT "player_mode_repeat"
+#define PLAYER_SETTINGS_MODE_SHUFFLE "player_mode_shuffle"
+#define PLAYER_SETTINGS_MODE_CONSUME "player_mode_consume"
+
 //#define DEBUG_PLAYER 1
 
 struct spk_enum
@@ -294,9 +300,10 @@ static struct commands_base *cmdbase;
 // from the player thread (where we can't use player_playback_pause)
 static int player_flush_pending;
 
-// Config values
+// Config values and player settings category
 static int speaker_autoselect;
 static int clear_queue_on_stop_disabled;
+static struct settings_category *player_settings_category;
 
 // Player status
 static enum play_status player_state;
@@ -594,7 +601,7 @@ metadata_update_queue_cb(void *arg)
       if (metadata->len_ms)
 	queue_item->song_length = metadata->len_ms;
 
-      ret = db_queue_update_item(queue_item);
+      ret = db_queue_item_update(queue_item);
       if (ret < 0)
 	DPRINTF(E_LOG, L_PLAYER, "Database error while updating queue with new metadata\n");
     }
@@ -2071,20 +2078,26 @@ playback_start_item(void *arg, int *retval)
   // device_shutdown_cb, since the unselected devices shouldn't have sessions.
   *retval = outputs_start(device_activate_cb, device_shutdown_cb, false);
   if (*retval < 0)
+    DPRINTF(E_WARN, L_PLAYER, "All selected speakers failed to start\n");
+
+  // autoselect also applies in non-error cases (if no devices were selected).
+  // Note that if session count is 0 then retval should never be positive,
+  // because that would mean we are initiating a session. Just to be safe, we
+  // check anyway.
+  if (speaker_autoselect && outputs_sessions_count() == 0 && *retval <= 0)
     {
-      DPRINTF(E_LOG, L_PLAYER, "All selected speakers failed to start\n");
-
-      // All selected devices failed, autoselect an unselected (if enabled)
-      for (device = outputs_list(); (*retval < 0) && speaker_autoselect && device; device = device->next)
+      for (device = outputs_list(); device; device = device->next)
 	{
-	  if (!device->selected && outputs_priority(device) != 0 && !device->session)
-	    *retval = outputs_device_start(device, device_activate_cb, false);
-	}
+	  if (device->selected || outputs_priority(device) == 0 || device->session)
+	    continue;
+
+	  *retval = outputs_device_start(device, device_activate_cb, false);
+	  if (*retval < 0)
+	    continue;
 
-      if (*retval >= 0)
-	{
 	  DPRINTF(E_LOG, L_PLAYER, "Autoselected %s device '%s'\n", device->type_name, device->name);
-	  outputs_device_select(device);
+	  outputs_device_select(device, -1);
+	  break;
 	}
     }
 
@@ -2450,6 +2463,33 @@ playback_pause(void *arg, int *retval)
   return COMMAND_END;
 }
 
+static enum command_state
+playback_flush(void *arg, int *retval)
+{
+  if (player_state == PLAY_STOPPED)
+    {
+      *retval = -1;
+      return COMMAND_END;
+    }
+
+  if (player_state == PLAY_PAUSED)
+    {
+      *retval = 0;
+      return COMMAND_END;
+    }
+
+  input_flush(NULL);
+
+  *retval = outputs_flush(device_flush_cb);
+  outputs_metadata_purge();
+
+  if (*retval > 0)
+    return COMMAND_PENDING; // async
+
+  // Otherwise we are done
+  return COMMAND_END;
+}
+
 static enum command_state
 playback_seek(void *arg, int *retval)
 {
@@ -2554,6 +2594,7 @@ speaker_set(void *arg, int *retval)
   struct speaker_set_param *speaker_set_param = arg;
   struct output_device *device;
   uint64_t *ids;
+  int max_volume;
   int nspk;
   int i;
 
@@ -2568,6 +2609,11 @@ speaker_set(void *arg, int *retval)
 
   DPRINTF(E_DBG, L_PLAYER, "Speaker set: %d speakers\n", nspk);
 
+  // Save the current master volume before we start selecting/unselecting, as
+  // that will affect master volume. See comment in outputs_device_select() for
+  // why we want to provide a max_volume.
+  max_volume = (player_state != PLAY_STOPPED) ? outputs_volume_get() : -1;
+
   for (device = outputs_list(); device; device = device->next)
     {
       for (i = 1; i <= nspk; i++)
@@ -2577,7 +2623,7 @@ speaker_set(void *arg, int *retval)
 	}
 
       if (i <= nspk)
-	outputs_device_select(device);
+	outputs_device_select(device, max_volume);
       else
 	outputs_device_deselect(device);
     }
@@ -2595,6 +2641,7 @@ speaker_enable(void *arg, int *retval)
 {
   uint64_t *id = arg;
   struct output_device *device;
+  int max_volume;
 
   *retval = -1;
 
@@ -2604,7 +2651,9 @@ speaker_enable(void *arg, int *retval)
 
   DPRINTF(E_DBG, L_PLAYER, "Speaker enable: '%s' (id=%" PRIu64 ")\n", device->name, *id);
 
-  outputs_device_select(device);
+  max_volume = (player_state != PLAY_STOPPED) ? outputs_volume_get() : -1;
+
+  outputs_device_select(device, max_volume);
 
   *retval = outputs_device_start(device, device_activate_cb, PLAYER_ONLY_PROBE);
 
@@ -2809,6 +2858,20 @@ speaker_authorize(void *arg, int *retval)
   return COMMAND_END;
 }
 
+static enum command_state
+speaker_start_all(void *arg, int *retval)
+{
+  outputs_stop_delayed_cancel();
+
+  *retval = outputs_start(device_activate_cb, device_shutdown_cb, false);
+
+  if (*retval > 0)
+    return COMMAND_PENDING; // async
+
+  // Otherwise we are done
+  return COMMAND_END;
+}
+
 static enum command_state
 volume_set(void *arg, int *retval)
 {
@@ -2935,6 +2998,9 @@ repeat_set(void *arg, int *retval)
 	return COMMAND_END;
     }
 
+  // Persist
+  SETTINGS_SETINT(player_settings_category, PLAYER_SETTINGS_MODE_REPEAT, repeat);
+
   *retval = 0;
   return COMMAND_END;
 }
@@ -2967,6 +3033,9 @@ shuffle_set(void *arg, int *retval)
   // Update shuffle mode
   shuffle = new_shuffle;
 
+  // Persist
+  SETTINGS_SETBOOL(player_settings_category, PLAYER_SETTINGS_MODE_SHUFFLE, shuffle);
+
  out:
   *retval = 0;
   return COMMAND_END;
@@ -2979,6 +3048,9 @@ consume_set(void *arg, int *retval)
 
   consume = cmdarg->intval;
 
+  // Persist
+  SETTINGS_SETBOOL(player_settings_category, PLAYER_SETTINGS_MODE_CONSUME, consume);
+
   *retval = 0;
   return COMMAND_END;
 }
@@ -3130,6 +3202,27 @@ player_playback_pause(void)
   return ret;
 }
 
+/**
+ * Flushes outputs and input buffer, but does not stop the input read loop. Used
+ * by the pipe input when a track change is registered. Flushing outputs will
+ * stop them, so the command is two-step, i.e. it starts them again.
+ *
+ * @return Returns 0 on success and a negative value on error
+ */
+
+int
+player_playback_flush(void)
+{
+  int ret;
+
+  ret = commands_exec_sync(cmdbase, playback_flush, NULL, NULL);
+  if (ret < 0)
+    return ret;
+
+  ret = commands_exec_sync(cmdbase, speaker_start_all, NULL, NULL);
+  return ret;
+}
+
 /**
  * Seeks to the position "seek_ms", depending on the given "seek_mode" seek_ms is
  * either the new position in the current track (seek_mode == PLAYER_SEEK_POSITION)
@@ -3514,8 +3607,6 @@ player(void *arg)
   if (!player_exit)
     DPRINTF(E_LOG, L_PLAYER, "Player event loop terminated ahead of time!\n");
 
-  db_speaker_clear_all();
-
   for (device = outputs_list(); device; device = device->next)
     {
       ret = db_speaker_save(device);
@@ -3540,8 +3631,13 @@ player_init(void)
   speaker_autoselect = cfg_getbool(cfg_getsec(cfg, "general"), "speaker_autoselect");
   clear_queue_on_stop_disabled = cfg_getbool(cfg_getsec(cfg, "mpd"), "clear_queue_on_stop_disable");
 
+  CHECK_NULL(L_PLAYER, player_settings_category = settings_category_get("player"));
+  ret = SETTINGS_GETINT(player_settings_category, PLAYER_SETTINGS_MODE_REPEAT);
+  repeat = (ret > 0) ? ret : REPEAT_OFF;
+  shuffle = SETTINGS_GETBOOL(player_settings_category, PLAYER_SETTINGS_MODE_SHUFFLE);
+  consume = SETTINGS_GETBOOL(player_settings_category, PLAYER_SETTINGS_MODE_CONSUME);
+
   player_state = PLAY_STOPPED;
-  repeat = REPEAT_OFF;
 
   CHECK_NULL(L_PLAYER, history = calloc(1, sizeof(struct player_history)));
 
